"templatetas for panoramas"
from django import template
from panorama.models import PanoramaInfo
from django.conf import settings
import panorama.settings as panoramasettings
register = template.Library()


@register.simple_tag
def panorama_js():
    "Javascript includes needed by panoramas"
    return """
    <script type="text/javascript" src="%spanorama/js/jquery.js"></script>
    <script type="text/javascript" 
            src="%spanorama/js/jquery.fancybox-1.3.1.pack.js"></script>
    <script type="text/javascript" 
            src="%spanorama/js/jquery.panorama.js"></script>
    """ % ((settings.STATIC_URL,)*3)


@register.inclusion_tag('panorama/img.html', takes_context=True)
def show_panorama(context, panorama):
    """Show a panorama.

       First argument must be a Panorama instance.

       Rest of arguments can be picked from context:
         - start_position
         - auto_start
         - viewport_width
         - speed
         - direction
         - control_display
         - mode_360
    """
    infos = []
    for subinfo in PanoramaInfo.__subclasses__():
        infos += subinfo.objects.filter(panoramainfo_ptr__panorama=panorama)

    out_context = {'panorama':panorama,
            'panoramainfos':infos,
            'STATIC_URL': settings.STATIC_URL}

    #Search display options in context, model and settings, in this order
    for opt in ('start_position', 'auto_start', 'viewport_width', 'speed',
                'direction', 'control_display', 'mode_360'):
        if context.get(opt) is not None:
            out_context[opt] = context.get(opt)
        elif getattr(panorama, opt, None) is not None:
            out_context[opt] = getattr(panorama, opt, None)
        else:
            out_context[opt] = getattr(panoramasettings, opt.upper())
    return out_context



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'PanoramaTranslation'
        db.create_table('panorama_panorama_translation', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=30)),
            ('language_code', self.gf('django.db.models.fields.CharField')(db_index=True, max_length=15, blank=True)),
            ('master', self.gf('django.db.models.fields.related.ForeignKey')(related_name='translations', to=orm['panorama.Panorama'])),
        ))
        db.send_create_signal('panorama', ['PanoramaTranslation'])

        # Adding unique constraint on 'PanoramaTranslation', fields ['language_code', 'master']
        db.create_unique('panorama_panorama_translation', ['language_code', 'master_id'])

        # Adding model 'Panorama'
        db.create_table('panorama_panorama', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('image', self.gf('django.db.models.fields.files.ImageField')(max_length=100)),
        ))
        db.send_create_signal('panorama', ['Panorama'])

        # Adding model 'PanoramaInfo'
        db.create_table('panorama_panoramainfo', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('panorama', self.gf('django.db.models.fields.related.ForeignKey')(related_name='panorama', to=orm['panorama.Panorama'])),
            ('x0', self.gf('django.db.models.fields.IntegerField')()),
            ('x1', self.gf('django.db.models.fields.IntegerField')()),
            ('y0', self.gf('django.db.models.fields.IntegerField')()),
            ('y1', self.gf('django.db.models.fields.IntegerField')()),
        ))
        db.send_create_signal('panorama', ['PanoramaInfo'])

        # Adding model 'PanoramaExternalLinkTranslation'
        db.create_table('panorama_panoramaexternallink_translation', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=30)),
            ('language_code', self.gf('django.db.models.fields.CharField')(db_index=True, max_length=15, blank=True)),
            ('master', self.gf('django.db.models.fields.related.ForeignKey')(related_name='translations', to=orm['panorama.PanoramaExternalLink'])),
        ))
        db.send_create_signal('panorama', ['PanoramaExternalLinkTranslation'])

        # Adding unique constraint on 'PanoramaExternalLinkTranslation', fields ['language_code', 'master']
        db.create_unique('panorama_panoramaexternallink_translation', ['language_code', 'master_id'])

        # Adding model 'PanoramaExternalLink'
        db.create_table('panorama_panoramaexternallink', (
            ('panoramainfo_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['panorama.PanoramaInfo'], unique=True, primary_key=True)),
            ('url', self.gf('django.db.models.fields.CharField')(max_length=200)),
        ))
        db.send_create_signal('panorama', ['PanoramaExternalLink'])

        # Adding model 'PanoramaNoteTranslation'
        db.create_table('panorama_panoramanote_translation', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=30)),
            ('description', self.gf('tinymce.models.HTMLField')(blank=True)),
            ('language_code', self.gf('django.db.models.fields.CharField')(db_index=True, max_length=15, blank=True)),
            ('master', self.gf('django.db.models.fields.related.ForeignKey')(related_name='translations', to=orm['panorama.PanoramaNote'])),
        ))
        db.send_create_signal('panorama', ['PanoramaNoteTranslation'])

        # Adding unique constraint on 'PanoramaNoteTranslation', fields ['language_code', 'master']
        db.create_unique('panorama_panoramanote_translation', ['language_code', 'master_id'])

        # Adding model 'PanoramaNote'
        db.create_table('panorama_panoramanote', (
            ('panoramainfo_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['panorama.PanoramaInfo'], unique=True, primary_key=True)),
        ))
        db.send_create_signal('panorama', ['PanoramaNote'])


    def backwards(self, orm):
        
        # Removing unique constraint on 'PanoramaNoteTranslation', fields ['language_code', 'master']
        db.delete_unique('panorama_panoramanote_translation', ['language_code', 'master_id'])

        # Removing unique constraint on 'PanoramaExternalLinkTranslation', fields ['language_code', 'master']
        db.delete_unique('panorama_panoramaexternallink_translation', ['language_code', 'master_id'])

        # Removing unique constraint on 'PanoramaTranslation', fields ['language_code', 'master']
        db.delete_unique('panorama_panorama_translation', ['language_code', 'master_id'])

        # Deleting model 'PanoramaTranslation'
        db.delete_table('panorama_panorama_translation')

        # Deleting model 'Panorama'
        db.delete_table('panorama_panorama')

        # Deleting model 'PanoramaInfo'
        db.delete_table('panorama_panoramainfo')

        # Deleting model 'PanoramaExternalLinkTranslation'
        db.delete_table('panorama_panoramaexternallink_translation')

        # Deleting model 'PanoramaExternalLink'
        db.delete_table('panorama_panoramaexternallink')

        # Deleting model 'PanoramaNoteTranslation'
        db.delete_table('panorama_panoramanote_translation')

        # Deleting model 'PanoramaNote'
        db.delete_table('panorama_panoramanote')


    models = {
        'panorama.panorama': {
            'Meta': {'object_name': 'Panorama'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100'})
        },
        'panorama.panoramaexternallink': {
            'Meta': {'object_name': 'PanoramaExternalLink', '_ormbases': ['panorama.PanoramaInfo']},
            'panoramainfo_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['panorama.PanoramaInfo']", 'unique': 'True', 'primary_key': 'True'}),
            'url': ('django.db.models.fields.CharField', [], {'max_length': '200'})
        },
        'panorama.panoramaexternallinktranslation': {
            'Meta': {'ordering': "('language_code',)", 'unique_together': "(('language_code', 'master'),)", 'object_name': 'PanoramaExternalLinkTranslation', 'db_table': "'panorama_panoramaexternallink_translation'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '15', 'blank': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'to': "orm['panorama.PanoramaExternalLink']"}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '30'})
        },
        'panorama.panoramainfo': {
            'Meta': {'object_name': 'PanoramaInfo'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'panorama': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'panorama'", 'to': "orm['panorama.Panorama']"}),
            'x0': ('django.db.models.fields.IntegerField', [], {}),
            'x1': ('django.db.models.fields.IntegerField', [], {}),
            'y0': ('django.db.models.fields.IntegerField', [], {}),
            'y1': ('django.db.models.fields.IntegerField', [], {})
        },
        'panorama.panoramanote': {
            'Meta': {'object_name': 'PanoramaNote', '_ormbases': ['panorama.PanoramaInfo']},
            'panoramainfo_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['panorama.PanoramaInfo']", 'unique': 'True', 'primary_key': 'True'})
        },
        'panorama.panoramanotetranslation': {
            'Meta': {'ordering': "('language_code',)", 'unique_together': "(('language_code', 'master'),)", 'object_name': 'PanoramaNoteTranslation', 'db_table': "'panorama_panoramanote_translation'"},
            'description': ('tinymce.models.HTMLField', [], {'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '15', 'blank': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'to': "orm['panorama.PanoramaNote']"}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '30'})
        },
        'panorama.panoramatranslation': {
            'Meta': {'ordering': "('language_code',)", 'unique_together': "(('language_code', 'master'),)", 'object_name': 'PanoramaTranslation', 'db_table': "'panorama_panorama_translation'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '15', 'blank': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'to': "orm['panorama.Panorama']"}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '30'})
        }
    }

    complete_apps = ['panorama']



"""
   Settings for panorama display.
"""
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

#VIEWPORT_WIDTH
#  Width of the panorama window
#
#  Value: width in pixels.
#  Default: 600
VIEWPORT_WIDTH = getattr(settings, 'PANORAMA_VIEWPORT_WIDTH', 600)

#SPEED
#  Speed of the panorama rotation.
#
#  Value: number; higher values means slower :-P
#  Default: 20000
SPEED = getattr(settings, 'PANORAMA_SPEED', 20000)

#DIRECTION
#  Starting direction of the rotation.
#
#  Value: ['left','right']
#  Default: 'left'
DIRECTION = getattr(settings, 'PANORAMA_DIRECTION', 'left')

#CONTROL_DISPLAY
#  Display rotation controls?
#
#  Value: ['auto', 'yes', 'no']
#  Default: 'auto'
CONTROL_DISPLAY = getattr(settings, 'PANORAMA_CONTROL_DISPLAY', 'auto')

#START_POSITION
#  Starting position of the panorama.
#
#  Value: position in pixels.
#  Default: 0
START_POSITION = getattr(settings, 'PANORAMA_START_POSITION', 0)

#AUTO_START
#  Start rotation automatically?
#
#  Value: [True, False]
#  Default: False
AUTO_START = getattr(settings, 'PANORAMA_AUTO_START', 'false')

#MODE_360
#  Loop over the image?
#
#  Value: [True, False]
#  Default: True
MODE_360 = getattr(settings, 'PANORAMA_MODE_360', 'true')

DIRECTION_CHOICES = (
    ('left', _('left')),
    ('right', _('right')),
)

CONTROL_DISPLAY_CHOICES = (
    ('auto', _('auto')),
    ('yes', _('yes')),
    ('no', _('no')),
)



"URL patterns for panorama"

from django.conf.urls.defaults import patterns, url
from django.views.generic import ListView, DetailView
from panorama.models import Panorama

urlpatterns = patterns('panorama.views',
  # Add url patterns here
  url(r'^$', ListView.as_view(model=Panorama), name='panorama_list'),
  url(r'^(?P<pk>\d+)$', DetailView.as_view(model=Panorama),
                        name='panorama_detail'),
  (r'^(?P<pk>\d+).json$', 'panorama_json'),
  (r'^note/(?P<pk>\d+)$', 'show_note'),
)



"""
Settings to be used when running unit tests
# python manage.py test --settings=panorama.test_settings panorama
"""

DATABASE_ENGINE = ''           # 'postgresql_psycopg2', 'postgresql', 'mysql',
                               # 'sqlite3' or 'oracle'.
DATABASE_NAME = ''             # Or path to database file if using sqlite3.
DATABASE_USER = ''             # Not used with sqlite3.
DATABASE_PASSWORD = ''         # Not used with sqlite3.
DATABASE_HOST = ''             # Set to empty string for localhost.
DATABASE_PORT = ''             # Set to empty string for default.

INSTALLED_APPS = (
    # Put any other apps that your app depends on here
    'panorama',
)
SITE_ID = 1

# This merely needs to be present - as long as your test case specifies a
# urls attribute, it does not need to be populated.
ROOT_URLCONF = ''



"""
Django-panorama
---------------

Integration of jQuery virtual tour in Django.

One model stores panoramic photo while a number of other models represents
clickable areas in the panorama. Included areas can show HTML block (using
tinymce) or display another url in a iframe.

gisa.buildingmaps integrates with this application giving one extra area to
allow navigating locations (virtual tour experience).
"""



"Admin interface for django-panorama"
from django.contrib import admin
from django import forms
from panorama.models import Panorama, ExternalLink, Note, Link
from django.core.urlresolvers import reverse
from multilingual.admin import (MultilingualModelAdmin,
                                MultilingualModelAdminForm)

class PanoramaWidget(forms.Select):
    "Select widget for panorama admin"
    class Media:
        js = ('panorama/js/jquery.js', 'panorama/js/panorama_admin.js',)

    def build_attrs(self, extra_attrs=None, **kwargs):
        attrs = super(PanoramaWidget, self).build_attrs(extra_attrs, **kwargs)
        json_url = reverse('panorama.views.panorama_json', kwargs={'pk':999999})
        attrs['onchange'] = "update_panorama_widget(this,map,'%s')" % json_url
        return attrs

class PanoramaInfoAdmin(admin.ModelAdmin):
    """
      Base ModelAdmin for models inheriting PanoramaInfo.

      Defines common values such as list_display and list_filter.
    """
    base_list_display = ('title', 'panorama',)
    base_list_filter = ('panorama',)

    def __init__(self, *args, **kwargs):
        self.list_display = self.base_list_display + self.list_display
        self.list_filter = self.base_list_filter + self.list_filter
        super(PanoramaInfoAdmin, self).__init__(*args, **kwargs)

class ExternalLinkForm(MultilingualModelAdminForm):
    "Admin form for Panorama ExternalLink"
    panorama = forms.ModelChoiceField(queryset=Panorama.objects.all(),
                                      widget=PanoramaWidget())
    class Meta:
        model = ExternalLink

class ExternalLinkAdmin(MultilingualModelAdmin, PanoramaInfoAdmin):
    "Administration of Panorama ExternalLinks"
    form = ExternalLinkForm
    #TODO: use_fieldests don't work in django 1.3-beta-1, watch multilingual_ng
    #use_fieldsets = (
    #        (None, {'fields': (('title', 'panorama', 'url'),),}),
    #        ('Advanced options', {
    #            'classes':('collapse',),
    #            'fields':(('x0', 'x1', 'y0', 'y1'),),
    #            }),
    #        )
    list_display = ('url',)

class NoteForm(MultilingualModelAdminForm):
    "Admin form for Panorama Note"
    panorama = forms.ModelChoiceField(queryset=Panorama.objects.all(),
                                      widget=PanoramaWidget())
    class Meta:
        model = Note

class NoteAdmin(MultilingualModelAdmin, PanoramaInfoAdmin):
    "Administration of Panorama Notes"
    form = NoteForm
    #use_fieldsets = (
    #        (None, {'fields': (('title',),
    #                           ('description',),
    #                           ('panorama' ),),}),
    #        ('Advanced options', {
    #            'classes':('collapse',),
    #            'fields':(('x0', 'x1', 'y0', 'y1'),),
    #            }),
    #        )

class LinkForm(forms.ModelForm):
    "Admin form for Panorama Link"
    panorama = forms.ModelChoiceField(queryset=Panorama.objects.all(),widget=PanoramaWidget())
    class Meta:
        model = Link

class LinkAdmin(admin.ModelAdmin):
    form = LinkForm
    fieldsets = (
            (None, {'fields': (('panorama', 'target_panorama'),),}),
            ('Advanced options', {
                'classes':('collapse',),
                'fields':(('x0','x1','y0','y1'),),
                }),
            )



#class PanoramaAdmin(MultilingualModelAdmin):
#    use_fieldsets = (
#            (None, {'fields': (('title', 'image'),),}),
#            ('Display options', {
#                'classes':('collapse',),
#                'fields':(('viewport_width',),
#                          ('speed',),
#                          ('direction',),
#                          ('control_display',),
#                          ('start_position',),
#                          ('auto_start',),
#                          ('mode_360',),
#                         ),
#            }),
#    )


#admin.site.register(Panorama, PanoramaAdmin)
admin.site.register(Panorama, MultilingualModelAdmin)
admin.site.register(ExternalLink, ExternalLinkAdmin)
admin.site.register(Note, NoteAdmin)
admin.site.register(Link, LinkAdmin)



"Views for django-panorama"
from django.shortcuts import render_to_response, get_object_or_404
from django.http import HttpResponse
from panorama.models import Panorama, Note
from django.template import RequestContext
import simplejson
# Create your views here.

def panorama_json(request, pk):
    "Return panorama info in json format"
    panorama = get_object_or_404(Panorama, pk=pk)
    mimetype = 'application/javascript'
    data_dict = {'imgurl': panorama.image.url,
                 'width': panorama.image.width,
                 'height': panorama.image.height}
    data = simplejson.dumps(data_dict)
    return HttpResponse(data, mimetype)

def show_note(request, pk):
    "Display panorama note for ajax"
    panoramanote = get_object_or_404(Note, pk=pk)
    context = RequestContext(request, {'panoramanote': panoramanote})
    return render_to_response('panorama/show_panoramanote.html', context)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Renaming model 'ExternalLink'
        db.rename_table('panorama_panoramaexternallink','panorama_externallink')

        # Renaming model 'Link'
        db.rename_table('panorama_panoramalink', 'panorama_link')

        # Renaming model 'ExternalLinkTranslation'
        db.rename_table('panorama_panoramaexternallink_translation',
                        'panorama_externallink_translation')

        # Renaming model 'Note'
        db.rename_table('panorama_panoramanote', 'panorama_note')

        # Renaming model 'NoteTranslation'
        db.rename_table('panorama_panoramanote_translation', 'panorama_note_translation')

    def backwards(self, orm):

        # Renaming model 'ExternalLink'
        db.rename_table('panorama_externallink','panorama_panoramaexternallink')

        # Renaming model 'Link'
        db.rename_table('panorama_link', 'panorama_panoramalink')

        # Renaming model 'ExternalLinkTranslation'
        db.rename_table('panorama_externallink_translation',
                        'panorama_panoramaexternallink_translation')

        # Renaming model 'Note'
        db.rename_table('panorama_note', 'panorama_panoramanote')

        # Renaming model 'NoteTranslation'
        db.rename_table('panorama_note_translation', 'panorama_panoramanote_translation')


    models = {
        'panorama.externallink': {
            'Meta': {'object_name': 'ExternalLink', '_ormbases': ['panorama.PanoramaInfo']},
            'panoramainfo_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['panorama.PanoramaInfo']", 'unique': 'True', 'primary_key': 'True'}),
            'url': ('django.db.models.fields.CharField', [], {'max_length': '200'})
        },
        'panorama.externallinktranslation': {
            'Meta': {'ordering': "('language_code',)", 'unique_together': "(('language_code', 'master'),)", 'object_name': 'ExternalLinkTranslation', 'db_table': "'panorama_externallink_translation'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '15', 'blank': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'to': "orm['panorama.ExternalLink']"}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '30'})
        },
        'panorama.link': {
            'Meta': {'object_name': 'Link', '_ormbases': ['panorama.PanoramaInfo']},
            'panoramainfo_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['panorama.PanoramaInfo']", 'unique': 'True', 'primary_key': 'True'}),
            'target_panorama': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['panorama.Panorama']"})
        },
        'panorama.note': {
            'Meta': {'object_name': 'Note', '_ormbases': ['panorama.PanoramaInfo']},
            'panoramainfo_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['panorama.PanoramaInfo']", 'unique': 'True', 'primary_key': 'True'})
        },
        'panorama.notetranslation': {
            'Meta': {'ordering': "('language_code',)", 'unique_together': "(('language_code', 'master'),)", 'object_name': 'NoteTranslation', 'db_table': "'panorama_note_translation'"},
            'description': ('tinymce.models.HTMLField', [], {'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '15', 'blank': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'to': "orm['panorama.Note']"}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '30'})
        },
        'panorama.panorama': {
            'Meta': {'object_name': 'Panorama'},
            'auto_start': ('django.db.models.fields.NullBooleanField', [], {'null': 'True', 'blank': 'True'}),
            'control_display': ('django.db.models.fields.CharField', [], {'max_length': '5', 'null': 'True', 'blank': 'True'}),
            'direction': ('django.db.models.fields.CharField', [], {'max_length': '5', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100'}),
            'mode_360': ('django.db.models.fields.NullBooleanField', [], {'null': 'True', 'blank': 'True'}),
            'speed': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'start_position': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'viewport_width': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'})
        },
        'panorama.panoramainfo': {
            'Meta': {'object_name': 'PanoramaInfo'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'panorama': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'panorama'", 'to': "orm['panorama.Panorama']"}),
            'x0': ('django.db.models.fields.IntegerField', [], {}),
            'x1': ('django.db.models.fields.IntegerField', [], {}),
            'y0': ('django.db.models.fields.IntegerField', [], {}),
            'y1': ('django.db.models.fields.IntegerField', [], {})
        },
        'panorama.panoramatranslation': {
            'Meta': {'ordering': "('language_code',)", 'unique_together': "(('language_code', 'master'),)", 'object_name': 'PanoramaTranslation', 'db_table': "'panorama_panorama_translation'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '15', 'blank': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'to': "orm['panorama.Panorama']"}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '30'})
        }
    }

    complete_apps = ['panorama']



"""
URL test patterns for panorama. Use this file to ensure a consistent
set of URL patterns are used when running unit tests. This test_urls
module should be referred to by your test class.
"""

from django.conf.urls.defaults import patterns, include


urlpatterns = patterns('panorama.views',
  (r'^panoramas/', include('panorama.urls')),
)



"Models for django-panorama"

from django.db import models
from multilingual.translation import TranslationModel
from django.utils.translation import ugettext_lazy as _
from tinymce.models import HTMLField
from panorama.settings import DIRECTION_CHOICES, CONTROL_DISPLAY_CHOICES

# Create your models here.

class Panorama (models.Model):
    """
      This model stores a panoramic image.
    """
    class Translation(TranslationModel):
        title = models.CharField(name=_('title'), max_length=30)
    image = models.ImageField(name=_('image'), upload_to='panoramas/')
    viewport_width = models.IntegerField(
            name=_('viewport width'),
            help_text=_("Width of the panorama window in pixels."),
            blank=True,
            null=True)
    speed = models.IntegerField(
            name=_('speed'),
            help_text=_("Speed of the panorama rotation. "
                        "Lower value means faster."),
            blank=True,
            null=True)
    direction = models.CharField(
            name=_('direction'),
            max_length=5,
            choices = DIRECTION_CHOICES,
            blank=True,
            null=True)
    control_display = models.CharField(
            name=_('control display'),
            help_text=_("Display rotation controls?"),
            max_length=5,
            choices = CONTROL_DISPLAY_CHOICES,
            blank=True,
            null=True)
    start_position = models.IntegerField(
            name=_('start position'),
            help_text=_("Start position of the panorama, in pixels"),
            blank=True,
            null=True)
    auto_start = models.NullBooleanField(
            name=_('auto start'),
            blank=True,
            null=True)
    mode_360 = models.NullBooleanField(
            name=_('mode 360'),
            help_text=_("Loop over the panorama?"),
            blank=True,
            null=True)

    class Meta:
        verbose_name = _('panorama')
        verbose_name_plural = _('panoramas')

    def __unicode__(self):
        return self.title or _('New Panorama')

    @models.permalink
    def get_absolute_url(self):
        return ('panorama_detail', [str(self.id)])


class PanoramaInfo(models.Model):
    """
      This base model defines the attributes needed to attach info to a
      panorama. Attached info will be seen as rectangles floating over
      panorama.
      
      Not really useful alone, see Note, Link, ExternalLink, etc.
    """
    panorama = models.ForeignKey('Panorama',
                                 name=_('panorama'),
                                 related_name = 'panorama')
    x0 = models.IntegerField(name=u'X0')
    x1 = models.IntegerField(name=u'X1')
    y0 = models.IntegerField(name=u'Y0')
    y1 = models.IntegerField(name=u'Y1')
    type = None
    url = None
    alt = None

    #class Meta:
    #    abstract = True

class ExternalLink(PanoramaInfo):
    """
      Show a external web inside a iframe when clicked in this
      PanoramaInfo.
    """
    class Translation(TranslationModel):
        title = models.CharField(name=_('title'), max_length=30)
    url = models.CharField(name=u'URL', max_length=200)
    type = 'iframe'

    class Meta:
        verbose_name = _('external link')
        verbose_name_plural = _('external links')

    def __unicode__(self):
        return self.title or _('New External Link')

    def alt(self):
        "Show title as img alt attribute"
        return self.title

class Link(PanoramaInfo):
    """
      Link to another Panorama model.
    """
    target_panorama = models.ForeignKey(Panorama, name=_("target panorama"))

    def url(self):
        return self.target_panorama.get_absolute_url()

    def alt(self):
        return self.target_panorama.title

    def __unicode__(self):
        return _(u"Link to panorama %s") % self.target_panorama.title

class Note(PanoramaInfo):
    """
      Show a HTML note as a popup when clicked this PanoramaInfo.
    """
    class Translation(TranslationModel):
        title = models.CharField(name=_('title'), max_length=30)
        description = HTMLField(name=_('description'), blank=True)
    type = 'ajax'

    class Meta:
        verbose_name = _('note')
        verbose_name_plural = _('notes')

    def __unicode__(self):
        return self.title or _('New Note')

    @models.permalink
    def get_absolute_url(self):
        return ('panorama.views.show_note', [str(self.id)])

    def alt(self):
        "Show title as img alt attribute"
        return self.title

    def url(self):
        "Url where this panoraminfo points"
        return self.get_absolute_url()



APP_NAME='panorama'
from setuptools import setup, find_packages
import os

version = '1.2'

setup(name='django-%s'%APP_NAME,
      version=version,
      description="Show panoramic photos in django",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='',
      author='Maider Likona',
      author_email='pisoni@gisa-elkartea.org',
      url='http://lagunak.gisa-elkartea.org/projects/djangopanorama',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      package_dir={
          APP_NAME: APP_NAME,
      },
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'PIL',
          'django-multilingual-ng',
          'django-tinymce',
          # -*- Extra requirements: -*-
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



"Tests for django-panorama"

from django.test import TestCase
from panorama.models import Panorama, Note, ExternalLink

class AppTestCase(TestCase):
    """
    Populate this class with unit tests for your application
    """

    fixtures = ('panorama',)
    
    urls = 'panorama.test_urls'
    
    def test_templatetag_show_panorama(self):
        "Test the show_panorama templatetag: valid panoramainfos"
        from panorama.templatetags.panoramas import show_panorama
        panorama = Panorama.objects.get(pk=1) #Get the 'Hall' panorama
        note = Note.objects.get(pk=1)
        note2 = Note.objects.get(pk=3)
        link = ExternalLink.objects.get(pk=2)
        request = {}
        context = show_panorama(request, panorama)
        panoramainfos = context['panoramainfos']
        self.assertEqual(len(panoramainfos), 2)
        self.assertTrue(note in panoramainfos)
        self.assertTrue(link in panoramainfos)
        self.assertFalse(note2 in panoramainfos)

    def test_panorama_detail(self):
        "Test the detail view of a panorama"
        response = self.client.get('/panoramas/1')
        self.assertEqual(response.status_code, 200)
        #Verify jquery.advancedpanorama is loaded
        self.assertContains(response, 'panorama/js/jquery.panorama.js')
        #Verify jquery.advancedpanorama is launched for this panorama
        self.assertContains(response, '$("img.advancedpanorama").panorama({')
        #Verify two panoramainfos are shown
        self.assertContains(response, '<area ', 2)
        #Verify one panoramainfos is iframe (externallink)
        self.assertContains(response, 'class="iframe"', 1)
        #Verify one panoramainfos is ajax (note)
        self.assertContains(response, 'class="ajax"', 1)

    def test_panorama_listing(self):
        "Test the listing view of panoramas"
        response = self.client.get('/panoramas/')
        self.assertEqual(response.status_code, 200)
        #Verify three panoramas are shown
        self.assertContains(response, '<a href="/panoramas/', 3)



"django-panorama templatetags for admin interface"
from django import template
from panorama.models import Panorama, PanoramaInfo
from django.core.urlresolvers import reverse
import re

register = template.Library()

@register.inclusion_tag('panorama/panorama_selector.html')
def panorama_selector(panoramainfo=None):
    "OpenLayers powered selector to set the bounding box of PanoramaInfos"
    if panoramainfo is None:
        panorama = Panorama.objects.all()[0]
    else:
        panorama = panoramainfo.panorama
    json_url = reverse('panorama.views.panorama_json',
                       kwargs={'pk':panorama.pk})
    return {'panorama':panorama,
            'panoramainfo':panoramainfo,
            'json_url':json_url}


class PanoramaInfoTypes(template.Node):
    "Add PanoramaInfo types to request"
    def __init__(self, var_name, *args, **kwargs):
        super(PanoramaInfoTypes, self).__init__(*args, **kwargs)
        self.var_name = var_name
    def render(self, request):
        infos = PanoramaInfo.__subclasses__()
        urls = []
        for info in infos:
            url = 'admin:%s_%s_add' % (info._meta.app_label,
                                       info._meta.module_name)
            urls.append({'label':info._meta.verbose_name, 'url':reverse(url)})
        request[self.var_name] = urls
        return ''

@register.tag
def panoramainfo_types(parser, token):
    "templatetag to load all diferent Models inherinting PanoramaInfo"
    match = re.search(r'panoramainfo_types as (\w+)', token.contents)
    if not match:
        raise template.TemplateSyntaxError, \
              "panoramainfo_types needs variable name. "\
              "Example {%panoramainfo_types as pitypes%}"
    var_name = match.groups()[0]
    return PanoramaInfoTypes(var_name)

